==> core-0.1.1.tgz: package/composer/execute.js <==
import * as remoteControl from "./remoteControl.js";
import path from "path";
import { commonFilePaths, format, writeFile } from "../files/utils.js";
import { detectOrCreateProject } from "../utils/create-project.js";
import { createOrUpdateFiles } from "../files/processors.js";
import { getPackageJson } from "../utils/common.js";
import { createEmptyWorkspace, populateWorkspaceDetails } from "../utils/workspace.js";
import { askQuestionsAndAssignValuesToWorkspace, ensureCorrectOptionTypes, prepareAndParseCliOptions } from "./options.js";

/**
 * @template {import("./config").ArgType} Args
 * @param {import("./config").ComposerConfig<Args>} config
 */
export async function executeComposer(config) {
    console.log("Executing " + config.metadata.package);

    const cliOptions = prepareAndParseCliOptions(config);

    const { remoteControlled, remoteControlOptions } = remoteControl.getStatus();
    let workingDirectory = "";
    if (!remoteControlled) {
        workingDirectory = determineWorkingDirectory(cliOptions);
        workingDirectory = await detectOrCreateProject(workingDirectory);
    } else {
        workingDirectory = remoteControlOptions.workingDirectory;
    }

    const workspace = createEmptyWorkspace();
    await populateWorkspaceDetails(cliOptions, workspace, workingDirectory);

    await askQuestionsAndAssignValuesToWorkspace(config, workspace, cliOptions);
    ensureCorrectOptionTypes(config, workspace);
    console.log(config.metadata.id, "options", workspace.options);

    const isInstall = true;

    await installPackages(config, workspace);
    await createOrUpdateFiles(config.files, workspace);
    await runHooks(config, workspace, isInstall);
}

/**
 *
 * @param {import("commander").OptionValues} options
 * @returns
 */
function determineWorkingDirectory(options) {
    let cwd = options.path ?? process.cwd();
    if (!path.isAbsolute(cwd)) {
        cwd = path.join(process.cwd(), cwd);
    }

    return cwd;
}

/**
 * @template {import("./config").ArgType} Args
 * @param {import("./config").ComposerConfig<Args>} config
 * @param {import("./config").Workspace<Args>} workspace
 */
export async function installPackages(config, workspace) {
    const content = await getPackageJson(workspace);

    for (const dependency of config.packages) {
        if (dependency.condition && !dependency.condition(workspace)) {
            continue;
        }

        if (dependency.dev) {
            if (!content.devDependencies) {
                content.devDependencies = {};
            }

            content.devDependencies[dependency.name] = dependency.version;
        } else {
            if (!content.dependencies) {
                content.dependencies = {};
            }

            content.dependencies[dependency.name] = dependency.version;
        }
    }

    const packageText = await format(workspace, commonFilePaths.packageJsonFilePath, JSON.stringify(content));
    await writeFile(workspace, commonFilePaths.packageJsonFilePath, packageText);
}

/**
 *
 * @template {import("./config").ArgType} Args
 * @param {import("./config").ComposerConfig<Args>} config
 * @param {import("./config").Workspace<Args>} workspace
 * @param {boolean} isInstall
 */
function runHooks(config, workspace, isInstall) {
    if (isInstall && config.installHook) config.installHook(workspace);
    else if (!isInstall && config.uninstallHook) config.uninstallHook(workspace);
}

/**
 *
 * @param {any} pkg
 * @returns {{id: string, package: string, version: string}}
 */
export function generateComposerInfo(pkg) {
    const name = pkg.name;
    const id = name.replace("@svelte-compose/composer-", "");

    return {
        id,
        package: name,
        version: pkg.version,
    };
}

==> core-0.1.5.tgz: package/composer/execute.js <==
import path from "path";
import { commonFilePaths, format, writeFile } from "../files/utils.js";
import { detectOrCreateProject } from "../utils/create-project.js";
import { createOrUpdateFiles } from "../files/processors.js";
import { getPackageJson } from "../utils/common.js";
import { createEmptyWorkspace, populateWorkspaceDetails } from "../utils/workspace.js";
import { askQuestionsAndAssignValuesToWorkspace, ensureCorrectOptionTypes, prepareAndParseCliOptions } from "./options.js";

/**
 * @template {import("./config").ArgType} Args
 * @param {import("./config").ComposerConfig<Args>} config
 * @param {import("./config").ComposerCheckConfig<Args>} checks
 * @param {import("./remoteControl.js").RemoteControlOptions} [remoteControlOptions]
 */
export async function executeComposer(config, checks, remoteControlOptions = undefined) {
    if (checks.preInstallation) {
        await runPreInstallationChecks(checks.preInstallation);
    }

    const remoteControlled = remoteControlOptions !== undefined;

    let cliOptions = {};
    let workingDirectory = "";
    if (!remoteControlled) {
        cliOptions = prepareAndParseCliOptions(config);
        workingDirectory = determineWorkingDirectory(cliOptions);
        workingDirectory = await detectOrCreateProject(workingDirectory);
    } else {
        cliOptions = remoteControlOptions.optionValues;
        workingDirectory = remoteControlOptions.workingDirectory;
    }

    const workspace = createEmptyWorkspace();
    await populateWorkspaceDetails(cliOptions, workspace, workingDirectory);

    await askQuestionsAndAssignValuesToWorkspace(config, workspace, cliOptions);
    ensureCorrectOptionTypes(config, workspace);

    const isInstall = true;

    await installPackages(config, workspace);
    await createOrUpdateFiles(config.files, workspace);
    await runHooks(config, workspace, isInstall);

    await runPostInstallationChecks(checks.postInstallation);
}

/**
 *
 * @param {import("commander").OptionValues} options
 * @returns
 */
function determineWorkingDirectory(options) {
    let cwd = options.path ?? process.cwd();
    if (!path.isAbsolute(cwd)) {
        cwd = path.join(process.cwd(), cwd);
    }

    return cwd;
}

/**
 * @template {import("./config").ArgType} Args
 * @param {import("./config").ComposerConfig<Args>} config
 * @param {import("./config").Workspace<Args>} workspace
 */
export async function installPackages(config, workspace) {
    const content = await getPackageJson(workspace);

    for (const dependency of config.packages) {
        if (dependency.condition && !dependency.condition(workspace)) {
            continue;
        }

        if (dependency.dev) {
            if (!content.devDependencies) {
                content.devDependencies = {};
            }

            content.devDependencies[dependency.name] = dependency.version;
        } else {
            if (!content.dependencies) {
                content.dependencies = {};
            }

            content.dependencies[dependency.name] = dependency.version;
        }
    }

    const packageText = await format(workspace, commonFilePaths.packageJsonFilePath, JSON.stringify(content));
    await writeFile(workspace, commonFilePaths.packageJsonFilePath, packageText);
}

/**
 *
 * @template {import("./config").ArgType} Args
 * @param {import("./config").ComposerConfig<Args>} config
 * @param {import("./config").Workspace<Args>} workspace
 * @param {boolean} isInstall
 */
function runHooks(config, workspace, isInstall) {
    if (isInstall && config.installHook) config.installHook(workspace);
    else if (!isInstall && config.uninstallHook) config.uninstallHook(workspace);
}

/**
 *
 * @param {any} pkg
 * @returns {{id: string, package: string, version: string}}
 */
export function generateComposerInfo(pkg) {
    const name = pkg.name;
    const id = name.replace("@svelte-compose/composer-", "");

    return {
        id,
        package: name,
        version: pkg.version,
    };
}

/**
 *
 * @param {import("./config").PreInstallationCheck[]} checks
 */
export async function runPreInstallationChecks(checks) {
    // console.log(checks);
}

/**
 * @template {import("./config").ArgType} Args
 * @param {import("./config").PostInstallationCheck<Args>[]} checks
 */
export async function runPostInstallationChecks(checks) {
    // console.log(checks);
}
